@@ -12,11 +12,6 @@
 
 /* LZX decompression implementation */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <mspack.h>
 #include <system.h>
 #include <lzx.h>
 
@@ -357,11 +352,12 @@ static unsigned char extra_bits[51];
 static void lzxd_static_init() {
   int i, j;
 
-  for (i = 0, j = 0; i < 51; i += 2) {
+  for (i = 0, j = 0; i < 50; i += 2) {
     extra_bits[i]   = j; /* 0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7... */
     extra_bits[i+1] = j;
     if ((i != 0) && (j < 17)) j++; /* 0,0,1,2,3,4...15,16,17,17,17,17... */
   }
+  extra_bits[50] = 17;
 
   for (i = 0, j = 0; i < 51; i++) {
     position_base[i] = j; /* 0,1,2,3,4,6,8,12,16,24,32,... */
