@@ -20,11 +20,6 @@
  * http://www.speakeasy.org/~russotto/quantumcomp.html
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <mspack.h>
 #include <system.h>
 #include <qtm.h>
 
