@@ -1,4 +1,4 @@
-/* cabextract 1.0 - a program to extract Microsoft Cabinet files
+/* cabextract 1.1 - a program to extract Microsoft Cabinet files
  * (C) 2000-2004 Stuart Caie <kyzer@4u.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -184,8 +184,28 @@ static int recall_file(struct file_mem *fml, char *name, char **from);
 static void forget_files(struct file_mem **fml);
 static int ensure_filepath(char *path);
 static char *cab_error(struct mscab_decompressor *cd);
-extern struct mspack_system cabextract_system;
 
+static struct mspack_file *cabx_open(struct mspack_system *this,
+				     char *filename, int mode);
+static void cabx_close(struct mspack_file *file);
+static int cabx_read(struct mspack_file *file, void *buffer, int bytes);
+static int cabx_write(struct mspack_file *file, void *buffer, int bytes);
+static int cabx_seek(struct mspack_file *file, off_t offset, int mode);
+static off_t cabx_tell(struct mspack_file *file);
+static void cabx_msg(struct mspack_file *file, char *format, ...);
+static void *cabx_alloc(struct mspack_system *this, size_t bytes);
+static void cabx_free(void *buffer);
+static void cabx_copy(void *src, void *dest, size_t bytes);
+
+/**
+ * A cabextract-specific implementation of mspack_system that allows
+ * the NULL filename to be opened for writing as a synonym for writing
+ * to stdout.
+ */
+static struct mspack_system cabextract_system = {
+  &cabx_open, &cabx_close, &cabx_read,  &cabx_write, &cabx_seek,
+  &cabx_tell, &cabx_msg, &cabx_alloc, &cabx_free, &cabx_copy, NULL
+};
 
 int main(int argc, char *argv[]) {
   int i, err;
@@ -368,7 +388,7 @@ static int process_cabinet(char *basename) {
 	    (unsigned char *) file->filename, (unsigned char *) args.dir,
 	    args.lower, isunix, file->attribs & MSCAB_ATTRIB_UTF_NAME)))
       {
-	fprintf(stderr, "out of memory!\n");
+	errors++;
 	break;
       }
 
@@ -387,17 +407,21 @@ static int process_cabinet(char *basename) {
       else {
 	if (args.pipe) {
 	  if (cabd->extract(cabd, file, NULL)) {
-	    fprintf(stderr, "stdout: %s\n", name, cab_error(cabd)); errors++;
+	    fprintf(stderr, "stdout: %s: %s\n", name,cab_error(cabd));
+	    errors++;
 	  }
 	}
 	else {
 	  if (!args.quiet) printf("  extracting %s\n", name);
+
 	  if (!ensure_filepath(name)) {
-	    fprintf(stderr, "%s: can't create file path\n", name); errors++;
+	    fprintf(stderr, "%s: can't create file path\n", name);
+	    errors++;
 	  }
 	  else {
 	    if (cabd->extract(cabd, file, name)) {
-	      fprintf(stderr, "%s: %s\n", name, cab_error(cabd)); errors++;
+	      fprintf(stderr, "%s: %s\n", name, cab_error(cabd));
+	      errors++;
 	    }
 	    else {
 	      set_date_and_perm(file, name);
@@ -639,7 +663,7 @@ static char *create_output_name(unsigned char *fname, unsigned char *dir,
   if (dir) x += strlen((char *) dir);
 
   if (!(name = malloc(x + 2))) {
-    fprintf(stderr, "out of memory!\n");
+    fprintf(stderr, "Can't allocate output filename (%u bytes)\n", x + 2);
     return NULL;
   }
   
@@ -677,6 +701,7 @@ static char *create_output_name(unsigned char *fname, unsigned char *dir,
      */
     do {
       if (fname >= fe) {
+	fprintf(stderr, "error in UTF-8 decode\n");
 	free(name);
 	return NULL;	
       }
@@ -727,6 +752,16 @@ static char *create_output_name(unsigned char *fname, unsigned char *dir,
       else if (lower)      c = (unsigned char) tolower((int) c);
     } while ((*p++ = c));
   }
+
+  /* search for "../" in cab filename part and change to "xx/".  This
+   * prevents any unintended directory traversal. */
+  for (p = &name[dir ? strlen(dir)+1 : 0]; *p; p++) {
+    if ((p[0] == '.') && (p[1] == '.') && (p[2] == '/')) {
+      p[0] = p[1] = 'x';
+      p += 2;
+    }
+  }
+
   return (char *) name;
 }
 
@@ -889,8 +924,6 @@ static char *cab_error(struct mscab_decompressor *cd) {
   return "unknown error";
 }
 
-
-
 struct mspack_file_p {
   FILE *fh;
   char *name;
@@ -1003,14 +1036,5 @@ static void cabx_free(void *buffer) {
 }
 static void cabx_copy(void *src, void *dest, size_t bytes) {
   memcpy(dest, src, bytes);
+  bcopy(src, dest, bytes);
 }
-
-/**
- * A cabextract-specific implementation of mspack_system that allows
- * the NULL filename to be opened for writing as a synonym for writing
- * to stdout.
- */
-static struct mspack_system cabextract_system = {
-  &cabx_open, &cabx_close, &cabx_read,  &cabx_write, &cabx_seek,
-  &cabx_tell, &cabx_msg, &cabx_alloc, &cabx_free, &cabx_copy, NULL
-};
